
// image.cc -- part of gkrell_sunlight
//
// This program is copyright (C) 2008 Mauro Persano, and is free
// software which is freely distributable under the terms of the
// GNU public license, included as the file COPYING in this
// distribution.  It is NOT public domain software, and any
// redistribution not permitted by the GNU General Public License is
// expressly forbidden without prior written permission from
// the author.

#include <string.h>
#include <errno.h>
#ifdef USE_PNG
#include <png.h>
#endif
#include <X11/xpm.h>

#include "image.h"
#include "panic.h"

#ifdef USE_PNG
image *
image::from_png(const char *file_name)
{
	FILE *fp;

	if (!(fp = fopen(file_name, "rb")))
		panic("failed to open `%s': %s", file_name, strerror(errno));

	png_structp png_ptr;

	if (!(png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0)))
		panic("png_create_read_struct");

	png_infop info_ptr;

	if (!(info_ptr = png_create_info_struct(png_ptr)))
		panic("png_create_info_struct");

	if (setjmp(png_jmpbuf(png_ptr)))
		panic("png error");

	png_init_io(png_ptr, fp);

	png_read_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, 0);

	int color_type = png_get_color_type(png_ptr, info_ptr);
	int bit_depth = png_get_bit_depth(png_ptr, info_ptr);

	if (color_type != PNG_COLOR_TYPE_RGB || bit_depth != 8)
		panic("invalid color type");

	int width = png_get_image_width(png_ptr, info_ptr);
	int height = png_get_image_height(png_ptr, info_ptr);

	image *p = new image(width, height);

	png_bytep *rows = png_get_rows(png_ptr, info_ptr);

	rgb *data = p->data;

	for (int i = 0; i < height; i++) {
		memcpy(data, rows[i], width*sizeof *data);
		data += width;
	}

	png_destroy_read_struct(&png_ptr, &info_ptr, 0);

	fclose(fp);

	return p;
}
#endif

static rgb
parse_xpm_color(const char *c_color)
{
	rgb r;

	if (*c_color == '#' && strlen(c_color) == 7) {
		for (int i = 0; i < 3; i++) {
			int v;
			sscanf(c_color + 1 + i*2, "%02x", &v);
			r.arr[i] = v;
		}
	}

	return r;
}

image *
image::from_xpm_data(char **xpm_data)
{
	XpmImage xpm_image = {0};

	if (XpmCreateXpmImageFromData(xpm_data, &xpm_image, 0) != XpmSuccess)
		panic("XpmCreateXpmImageFromBuffer failed");

	image *p = new image(xpm_image.width, xpm_image.height);

	rgb *dest = p->data;

	const unsigned *end = &xpm_image.data[xpm_image.width*xpm_image.height];

	for (const unsigned *q = xpm_image.data; q != end; q++)
		*dest++ = parse_xpm_color(xpm_image.colorTable[*q].c_color);

	XpmFreeXpmImage(&xpm_image);

	return p;
}

void
image::to_ppm(FILE *fp) const
{
	fprintf(fp, "P6\n%d %d\n255\n", width, height);

	for (rgb *p = data; p != &data[width*height]; p++)
		fprintf(fp, "%c%c%c", p->r, p->g, p->b);
}
