
// sunlight_map.h -- part of gkrell_sunlight
//
// This program is copyright (C) 2008 Mauro Persano, and is free
// software which is freely distributable under the terms of the
// GNU public license, included as the file COPYING in this
// distribution.  It is NOT public domain software, and any
// redistribution not permitted by the GNU General Public License is
// expressly forbidden without prior written permission from
// the author.

#ifndef SUNLIGHT_MAP_H_
#define SUNLIGHT_MAP_H_

#include <stdio.h>
#include <time.h>

#include <vector>

#include "image.h"
#include "vector3.h"

class
sunlight_map
{
  public:
  	sunlight_map(int width);
	virtual ~sunlight_map();

	int get_height() const { return cur_height; }
	int get_width() const { return cur_width; }
	rgb *get_rgb_buffer() { return sunlight_image->data; }
	void set_width(int width);
	void update(time_t t);
	void to_ppm(FILE *out);

  private:
	vector3 sun_vector(int julian_date, int gmt_seconds) const;

	image *earth_texture;
	int cur_width;
	int cur_height;
	image *sunlight_image;

	void initialize_sunlight_image();

	std::vector<float> phi_sin, phi_cos, theta_sin, theta_cos; 

	// disabled
	sunlight_map(const sunlight_map&);
	sunlight_map& operator=(const sunlight_map&);
};

#endif // SUNLIGHT_MAP_H_
